<<<<<<< HEAD
# System API Analysis: PrivacyFX

This is an extension of [SootFX](https://github.com/secure-software-engineering/SootFX/tree/main), which is a static code feature extraction tool for Java and Android. It is built to be extended with new resource providers and feature extraction units. SootFX extracts method, class and whole-program features for jar and apk files, for apks it also extracts manifest features. PrivacyFX can be used in the following two modes:

1. Privacy Source Detection: This mode can be used to take a dataset of methods and their description as input. It then finds those methods in the given app. The script _findSystemPrivacySources.py_ extracts privacy-related system API sources from the given Android app.

2. Identifier Dataset Construction: The script _collectPrivacyRelevantDataset.py_ extracts system API methods which we categorize as potential sources of privacy-related data.

# Privacy Source Detection

This mode matches the API calls within an APK with the identifier API dataset, which is desribed in the input file _input/IdentifierAPIDataset.txt_. This dataset segregates data sources into the following risk categories:

1. **Risk 1 directly identifiable data sources:**
System API methods which collect identifiable personal data (eg. IP address, SSN, IBAN) in the background. These are further labelled as device or other IDs and personal information.

2. **Risk  partially identifiable data sources:**
System API methods which collect partially identifiable data (eg. pincode, family name) in the background. These are further labelled as personal information, device, location, browsing, calendar, app activity, audio, photos and videos and session data.

3. **Risk 3 access data sources:**
System API methods which collect data used to grant access to the user (eg. username, password). These methods may not directly or even indirectly identify the user, but still need to be considered sensitive and risky.

4. **Risk 4 context-dependent sources:**
System API methods which cannot be categorised without knowing the context and content of the data they return (eg. read, recordAudio).

Running this mode:

1) Use the feature _WholeProgramPrivateSourcesAPICallCount_ in the config.yaml file _(input/config.yaml)_ and add the path to the input file _(input/IdentifierAPIDataset.txt)_. Both these files can also be generated automatically by running the script _input/prepareInputFile.py_.

2) Run the script _findSystemPrivacySources.py_ which executes the PrivacyFX jar file iteratively and summarises the results in the output folder.

# Identifier Dataset Construction

This mode constructs the identifier API dataset used in mode 1. In this mode, PrivacyFX detects system API getter methods (from Android, Java and third party libraries) which may be sources of privacy-related data.

PrivacyFX crawls the static call graph to look for the presence of Android and Java API methods and privacy-relevant third-party libraries which have been taken from the (AppBrain)[https://www.appbrain.com/] rankings. These third-party libraries can be broadly classified into the following criteria:

1. **Analytics methods:** 
Methods which allow developers track and monitor users' engagement with their apps (top libraries taken from AppBrain and other literature). Eg. Google Analytics, Firebase Analytics, Moat, Amplitude, AppsFlyer, Adjust, Flurry, CrashLytics.

2. **Advertisements:** 
Methods responsible for grabbing users' attention but more often are a source of frustration for the users (top libraries taken from AppBrain and other literature). Eg. Google Ads, Facebook Ads, Fyber, Applovin, Mopub, Admob and Open Measurement.

4. **Authentication**:
Methods that give user access to a system, email, network etc. Eg. Google authentication, Apache network authentication.

5. **IO and Data Serialization:**
Methods for handling input output operations. Eg. Google GSON, Google Protobuf, SLF4J, Apache Commons and Apache Derby.

6. **Network:** 	
Network related methods. Eg. Apache HTTP.

7. **Email:**
Methods that handle sending, receiving and processing of Emails. Eg. Apache James.

8. **Image Processing**:
Methods that detect and process an image. These can be used to scan barcodes or for other purposes. Eg. Google zxing.

9. **Other Libraries:** 
Other third-party libraries like Baidu (location), Joda time, Apache Harmony, Google i18n (phone numbers).

Running this mode:

1) Use the feature _ConstructPrivacyRelevantDataset_ in the config.yaml file _(input/config.yaml)_. Note: no need to add any input file in this case.

2) Run the script _collectPrivacyRelevantDataset.py_.

3) The possible sources are categorised as _account, bluetooth, location, device, audio, media, database, network, email, wifi, authentication, advertisement, ads and analytics, analytics, data serialization, image loaders, image processing and logging methods_ in the folder _possibledataset_.


# Usage
PrivacyFX can be used as CLI.
The jar PrivacyFX.jar can be used as follows:
    
    ```
    java -jar PrivacyFX.jar "path/to/apk" "path/to/out/" "path/to/config.yaml" "path/to/Android/sdk/platforms"
    ```
