import os
import subprocess
import json
import csv

def item_generator(json_input, lookup_key):
    if isinstance(json_input, dict):
        for k, v in json_input.items():
            if k.lower() == lookup_key.lower():
                yield v
            else:
                yield from item_generator(v, lookup_key)
    elif isinstance(json_input, list):
        for item in json_input:
            yield from item_generator(item, lookup_key)

def find_all_files(folder,ext):
    ext_files = []
    for root, dirs, files in os.walk(folder):
        for file in files:
            if file.endswith(f'.{ext}'):
                ext_files.append(os.path.join(root, file))
    return ext_files, len(ext_files)


def extractAppName(name: str):
    domain = name.split('_')[0]
    app_name = name.split('_')[1].split('.')[0]

    return domain,app_name

def readOtherDatafromFile(input_file):
    out_dict={}
    f = open(input_file,'r')
    for x in f:
        methodname=x.split('#')[0]
        data=x.split('#')[1]
        if(data != '\n'):
            #print(methodname, data)
            out_dict[methodname]=data
    return out_dict


def writeToExcel(csv_file, excel_file):
    import pandas as pd
    #read_file_product = pd.read_csv(csv_file)
    #read_file_product.to_excel(excel_file, index = None, header = True)
    new_dataFrame = pd.read_csv(csv_file)
    new_excel = pd.ExcelWriter(excel_file)
    new_dataFrame.to_excel(new_excel, index=False)
    new_excel.save()


def initialiseStatsCSVFile(name: str):
    header = ['App name','Total Sources','Risk 1 identifiers ','Risk 1 sources','Risk 2 identifiers','Risk 2 sources', 'Risk 3 identifiers', 'Risk 3 sources','Risk 4 identifiers', 'Risk 4 sources','Risk 1 permissions', 'Risk 2 permissions','Risk 3 permissions']
    with open(name, 'w', encoding='UTF8') as f:
        writer = csv.writer(f)
        # write the header
        writer.writerow(header)

def initialiseTPLCSVFile(name:str):
    header = ['App name','Risk 1 Method Libraries', 'Risk 2 Method Libraries', 'Risk 3 Method Libraries', 'Risk 4 Method Libraries']
    with open(name, 'w', encoding='UTF8') as f:
        writer = csv.writer(f)
        # write the header
        writer.writerow(header)

def initialiseIdentifierStatsCSVFile(name: str):
    header = ['Label','Identifier','Apps','Percentage']
    with open(name, 'w', encoding='UTF8') as f:
        writer = csv.writer(f)
        # write the header
        writer.writerow(header)

current_dir = os.path.dirname(os.path.abspath(__file__))

#All the necessary input files
output_dir = f'{current_dir}/output/RQ1/AllRQ1Apps'
stats_file = f'{current_dir}/stats/RQ1/AllApps/riskdistributionrandom_stats.csv'
tpl_file = f'{current_dir}/stats/RQ1/AllApps/tplrandom_stats.csv'
identifierstats_file = f'{current_dir}/stats/RQ1/AllApps/identifier_stats.csv'
excel_file = f'{current_dir}/stats/RQ1/AllApps/result_stats.xlsx'

#Initialize stats files
initialiseStatsCSVFile(stats_file)
initialiseTPLCSVFile(tpl_file)
initialiseIdentifierStatsCSVFile(identifierstats_file)
tpl_dict = readOtherDatafromFile(f'{current_dir}/input/LibraryMethods.txt')
perm_dict = readOtherDatafromFile(f'{current_dir}/input/Permissions.txt')

#Collect all json files from the given directory
json_files, totalapps = find_all_files(output_dir, 'json')
print("Total apps",totalapps)

if json_files is not None:
 #   risk1domaindict = {"ecom":[], "health":[], "chat":[], "game":[], "fin":[], "media":[], "tech":[], "art":[], "auto":[], "beauty":[], "dating":[], "comics":[], "food":[], "house":[], "maps":[], "photo":[], "tools":[], "random":[]}
  #  risk2domaindict = {"ecom":[], "health":[], "chat":[], "game":[], "fin":[], "media":[], "tech":[], "art":[], "auto":[], "beauty":[], "dating":[], "comics":[], "food":[], "house":[], "maps":[], "photo":[], "tools":[], "random":[]}
   # risk3domaindict = {"ecom":[], "health":[], "chat":[], "game":[], "fin":[], "media":[], "tech":[], "art":[], "auto":[], "beauty":[], "dating":[], "comics":[], "food":[], "house":[], "maps":[], "photo":[], "tools":[], "random":[]}
   # risk4domaindict = {"ecom":[], "health":[], "chat":[], "game":[], "fin":[], "media":[], "tech":[], "art":[], "auto":[], "beauty":[], "dating":[], "comics":[], "food":[], "house":[], "maps":[], "photo":[], "tools":[], "random":[]}
    tmp = {"risk4":[]}
    labels_in_apps = {"Risk 1 Identifiable personal information":[], "Risk 1 Identifiable device or other IDs":[], "Risk 2 Partially identifiable personal information":[], "Risk 2 Partially identifiable audio data":[], "Risk 2 Partially identifiable calendar data":[], "Risk 2 Partially identifiable browsing data":[], "Risk 2 Partially identifiable app activity data":[], "Risk 2 Partially identifiable Photos and videos data":[], "Risk 2 Partially identifiable location data":[], "Risk 2 Partially identifiable device data":[], "Risk 2 Partially identifiable session data":[], "Risk 3 Authentication data":[], "Risk 3 Email Authentication data":[], "Risk 3 Network Authentication data":[], "Risk 4 Context dependent UI data":[], "Risk 4 Context dependent photos and videos data":[], "Risk 4 Context dependent audio data":[], "Risk 4 Context dependent email data":[], "Risk 4 Context dependent database":[], "Risk 4 Context dependent SMS data":[], "Risk 4 Context dependent calendar data":[]}
    for f in json_files:
        array = f.split('/')
        filename = array.pop()
        app_name = filename.split('.apk.json')[0]
        with open(f) as ff:
            risk1methods = 0
            risk1identifiers = 0
            risk1_tplmethods = set()
            risk1permissions = []
            risk2methods = 0
            risk2identifiers = 0
            risk2_tplmethods = set()
            risk2permissions = []
            risk3methods = 0
            risk3identifiers = 0
            risk3_tplmethods = set()
            risk3permissions = []
            risk4methods = 0
            risk4identifiers = 0
            risk4_tplmethods = set()
            totalmethods = 0
            Data = json.load(ff)
            for k,v in Data.items():
                if k.startswith('Risk 1'):
                    for kv in v:
                        risk1identifiers = risk1identifiers + 1
                        risk1methods = risk1methods + len(v[kv])
                        for method in v[kv]:
                            if method in tpl_dict:
                                risk1_tplmethods.add(tpl_dict[method].rstrip())
                            if method in perm_dict:
                                risk1permissions.append(perm_dict[method].rstrip())
                    #if app_name.startswith('com') and 
                    if app_name not in labels_in_apps[k]:
                        labels_in_apps[k].append(app_name)

                elif k.startswith('Risk 2'):
                    for kv in v:
                        risk2identifiers = risk2identifiers + 1
                        risk2methods = risk2methods + len(v[kv])
                        for method in v[kv]:
                            if method in tpl_dict:
                                risk2_tplmethods.add(tpl_dict[method].rstrip())
                            if method in perm_dict:
                                risk2permissions.append(perm_dict[method].rstrip())
                    #if app_name.startswith('com') and 
                    if app_name not in labels_in_apps[k]:
                        labels_in_apps[k].append(app_name)
                    
                elif k.startswith('Risk 3'):
                    for kv in v:
                        risk3identifiers = risk3identifiers + 1
                        risk3methods = risk3methods + len(v[kv])
                        for method in v[kv]:
                            if method in tpl_dict:
                                risk3_tplmethods.add(tpl_dict[method].rstrip())
                            if method in perm_dict:
                                risk3permissions.append(perm_dict[method].rstrip()) 
                    #if app_name.startswith('com') and 
                    if app_name not in labels_in_apps[k]:
                        labels_in_apps[k].append(app_name)
                    
                elif k.startswith('Risk 4'):
                    if app_name not in tmp['risk4']:
                        tmp['risk4'].append(app_name)
                    for kv in v:
                        risk4identifiers = risk4identifiers + 1
                        risk4methods = risk4methods + len(v[kv])
                        for method in v[kv]:
                            if method in tpl_dict:
                                risk1_tplmethods.add(tpl_dict[method].rstrip())
                    #if app_name.startswith('com') and 
                    if app_name not in labels_in_apps[k]:
                        labels_in_apps[k].append(app_name)
            
        totalmethods = risk1methods + risk2methods + risk3methods + risk4methods
        stats = [app_name,totalmethods,risk1identifiers,risk1methods,risk2identifiers,risk2methods,risk3identifiers,risk3methods,risk4identifiers,risk4methods,risk1permissions,risk2permissions,risk3permissions]
        with open(stats_file, 'a', encoding='UTF8', newline='') as f:
            writer = csv.writer(f)
            writer.writerow(stats)
        tpl_stats = [app_name,list(risk1_tplmethods),list(risk2_tplmethods),list(risk3_tplmethods),list(risk4_tplmethods)]
        with open(tpl_file, 'a', encoding='UTF8', newline='') as f:             
            writer = csv.writer(f)
            writer.writerow(tpl_stats)
print(len(labels_in_apps))
for k in labels_in_apps:
    print(k,"sources found in",len(labels_in_apps[k]),"apps")
for k in tmp:
    print(k,"sources found in",len(tmp[k]),"apps")

'''
print("Summary of identifier labels in apps:")
for k in labels_in_apps:
    ecom_src = 0
    chat_src = 0
    fin_src = 0
    game_src = 0
    tech_src = 0
    health_src = 0
    media_src = 0
    others_src = 0
    print(k,"sources found in",len(labels_in_apps[k]),"apps-")
    for v in labels_in_apps[k]:
        if v.startswith('ecom'):
            ecom_src = ecom_src + 1
        elif v.startswith('chat'):
            chat_src = chat_src + 1
        elif v.startswith('fin'):
            fin_src = fin_src + 1
        elif v.startswith('game'):
            game_src = game_src + 1
        elif v.startswith('tech'):
            tech_src = tech_src + 1
        elif v.startswith('health'):
            health_src = health_src + 1
        elif v.startswith('media'):
            media_src = media_src + 1
        else:
            others_src = others_src + 1
    print("E-commerce and shopping:",ecom_src)
    print("Messaging and social media:",chat_src)
    print("Banking and finance:",fin_src)
    print("Sports and gaming:",game_src)
    print("Technology and education:",tech_src)
    print("Health and fitness:",health_src)
    print("News and entertainment:",media_src)
    print("Others:",others_src)

print('Domain wise risk distribution')
print('Risk 1:')
print(risk1domaindict)
print('Risk 2:')
print(risk2domaindict)
print('Risk 3:')
print(risk3domaindict)
print('Risk 4:')
print(risk4domaindict)
'''
print("Total apps:", totalapps)
allidentifiersdict = {"Risk 1 Personal Information": ["User ID", "Email address", "Client ID", "Phone number"], "Risk 1 Device or other IDs": ["IP Address", "MAC Address", "Session ID", "SIM Card information", "Device ID", "BSSID", "Facebook attribution ID", "Advertiser ID", "Cell ID", "Bluetooth information", "Application ID", "Call ID", "Zone ID", "IMEI", "MEID", "Audio device information", "Network access identifier"], "Risk 2 Personal Information": ["Name", "Profile photo", "Postal address", "Birthday", "Photo URL","Account", "User data"], "Risk 2 Audio data": ["Audio information"], "Risk 2 Browsing data": ["HTTP Cookie", "Browser Cookie", "Web browsing history", "HTTP response history"], "Risk 2 Location data": ["Approximate location", "Location provider", "Location area code"], "Risk 2 App activity": ["App activity", "App interactions", "Installed apps"], "Risk 2 Photos and videos": ["QR Code"], "Risk 2 Calendar": ["Date or time"], "Risk 2 Session data": ["User agent"], "Risk 2 Device data": ["Bluetooth information", "Mobile network information", "SIM card information", "Primary scrambling code", "Mobile network code", "Mobile country code"], "Risk 3 authentication": ["Username", "Password", "Token", "User data"], "Risk 4 Context dependent data": ["Text field", "Photo", "Message", "Audio", "Calendar entry", "Email"]}
for l in allidentifiersdict:
    print(l)
    with open(identifierstats_file, 'a', encoding='UTF8', newline='') as f:
            writer = csv.writer(f)    
            writer.writerow([l,"","",""])
    for ll in allidentifiersdict[l]:
        apps = set()
        if json_files is not None:
            for f in json_files:
                array = f.split('/')
                filename = array.pop()
                app_name = filename.split('.apk.json')[0]
                with open(f) as ff:
                    Data = json.load(ff) 
                    for i in item_generator(Data, ll):
                        apps.add(app_name)
        print(ll,":",len(apps),"apps in total.")
        print(round((len(apps)/totalapps)*100,2),'% of all apps.')
        with open(identifierstats_file, 'a', encoding='UTF8', newline='') as f:
            writer = csv.writer(f)    
            writer.writerow(["",ll,len(apps),round((len(apps)/totalapps)*100,2)])
#writeToExcel(identifierstats_file, excel_file)
'''
while True:
    val = input("What would you like to search? Type quit to exit.")
   # data_map =  {"ecom":[], "health":[], "chat":[], "game":[], "fin":[], "media":[], "tech":[], "art":[], "auto":[], "beauty":[], "dating":[], "comics":[], "food":[], "house":[], "maps":[], "photo":[], "tools":[],"random":[]}
    apps = set()
    if val == 'quit':
        break
    if json_files is not None:
        for f in json_files:
            array = f.split('/')
            filename = array.pop()
            app_name = filename.split('.apk.json')[0]
            with open(f) as ff:
                Data = json.load(ff) 
                for i in item_generator(Data, val):
                    print('App',app_name,'collects',val)
                    #data_map[domain].append(app_name)
                    apps.add(app_name)
    print(len(apps),'apps collect',val)
    #print(data_map)
#TODO: Among all personal information sources, how many apps have risk 1, how many have risk 2 and how many have common.
#Device data: risk 1 and risk 2
#Personal information : risk 1 and 2
#Audio data: risk 2 and risk 4
'''
