pmport json
import subprocess
import os
import time
import csv

#This script checks for privacy-relevant system API calls in Android apps. It uses an extension of SootFX, a static analysis based feature extraction tool.
def find_all_files(folder,ext):
    ext_files = []
    for root, dirs, files in os.walk(folder):
        for file in files:
            if file.endswith(f'.{ext}'):
                ext_files.append(os.path.join(root, file))
    return ext_files

def extractAppName(name: str):
    domain = name.split('.')[0]
    app_name = name.split('.')[1]
    ext = name.split('.')[2]
    return domain,app_name

def initialiseCSVFile(name: str):
    header = ['app', 'domain', 'processing time','post processing time']
    with open(name, 'w', encoding='UTF8') as f:
        writer = csv.writer(f)
        # write the header
        writer.writerow(header)

def item_generator(json_input, lookup_key): 
    if isinstance(json_input, dict):
        for k, v in json_input.items():
            if k == lookup_key:
                yield v
            else:
                yield from item_generator(v, lookup_key)
    elif isinstance(json_input, list):
        for item in json_input:
            yield from item_generator(item, lookup_key)

current_dir = os.path.dirname(os.path.abspath(__file__))
script_start_time = time.perf_counter()
#Pre processing:
#Set the Android platforms file, the APK folder and the output file

#Remote path
prefix_path = '/data/PRICEExperiments/'

#Local path


platforms_dir = f'{current_dir}/platforms'
apk_dir = f'{prefix_path}ECommerceAndShopping'
#apk_dir = f'{prefix_path}MessagingAndSocialMedia'
#apk_dir = f'{prefix_path}BankingAndFinance'
#apk_dir = f'{prefix_path}SportsAndGames'
#apk_dir = f'{prefix_path}TechAndEducation'
#apk_dir = f'{prefix_path}HealthAndFitness'
#apk_dir = f'{prefix_path}NewsAndEntertainment'
#apk_dir = f'{prefix_path}Others'

#Get the path of the directory containing this script
input_dir = f'{current_dir}/input'
output_dir = f'{current_dir}/output/constructPrivacyRelevantDataset'
apk_files = find_all_files(apk_dir, 'apk')
csv_file=f'{current_dir}/stats/constructPrivacyRelevantDataset/ecom_stats.csv'
initialiseCSVFile(csv_file)

#Get all APK files in the given folder and iterate over them
if apk_files is not None:
    for f in apk_files:
        #print(f)
        domain, app_name = extractAppName(os.path.basename(f))
        app_folder = domain+"_"+app_name
        new_dir = f'{output_dir}/{app_folder}'
        #print(new_dir)
        config_files = find_all_files(input_dir, 'yaml')
        if config_files is not None:
            for c in config_files:
                config_file=c
        else:
            print('Error: no config file given as input.')
        #subprocess.run(["mkdir", new_dir])
        print('....................................Started processing the application:',app_name,'from the domain:',domain,' ....................................')
        #Processing: 
        #Use PrivacyFX to extract all risky system API sources.
        process_start_time = time.perf_counter()
        print("Command executed:")
        print("java -jar PrivacyFX.jar "+f +" " +current_dir + "/ "+config_file +" "+platforms_dir)
        os.system("java -jar PrivacyFX.jar "+f +" " +current_dir + "/ "+config_file +" "+platforms_dir
        process_end_time = time.perf_counter()
        print('....................................Finished processing the application:',app_name,'from the domain:',domain,' ....................................')
        print('Time taken:',process_end_time - process_start_time,' seconds')
        postprocess_start_time = time.perf_counter()
        #Post processing: 
        #Sort the output json file to order the risk categories according to their danger
        jsonOutput = f'{current_dir}/output/AndroidAPIMethods.json'
        orderedJsonOutput = f'{output_dir}/{domain}_{app_name}.json'
        with open(jsonOutput) as f:
            json_data = json.load(f)
        sorted_json_data = json.dumps(json_data, sort_keys=True, indent=2)
        with open(orderedJsonOutput, "w") as outfile:
            outfile.write(sorted_json_data)
        subprocess.run(["rm", jsonOutput])
        val = False
        with open(orderedJsonOutput) as f:
            Data= json.load(f) 
        postprocess_end_time = time.perf_counter()
        data=[app_name, domain, process_end_time - process_start_time, postprocess_end_time - postprocess_start_time]#, val]
        subprocess.run(["rm", "wp.csv"])
        #Write the statistics
        with open(csv_file, 'a', encoding='UTF8', newline='') as f:
            writer = csv.writer(f)
            writer.writerow(data)
            
else:
    print('Error: no APKs given as input.')

script_end_time = time.perf_counter()
print('Script duration: ',(script_end_time - script_start_time)/60,'minutes')
