import json
import subprocess
import os
import time
import csv

#This script checks for privacy-relevant system API calls in Android apps. It uses an extension of SootFX, a static analysis based feature extraction tool.
def find_all_files(folder,ext):
    ext_files = []
    for root, dirs, files in os.walk(folder):
        for file in files:
            if file.endswith(f'.{ext}'):
                ext_files.append(os.path.join(root, file))
    return ext_files

def extractAppName(name: str):
    domain = name.split('.')[0]
    app_name = name.split('.')[1]
    ext = name.split('.')[2]
    return domain,app_name

def initialiseCSVFile(name: str):
    header = ['app', 'processing time','post processing time']
    with open(name, 'w', encoding='UTF8') as f:
        writer = csv.writer(f)
        # write the header
        writer.writerow(header)

def item_generator(json_input, lookup_key):
    if isinstance(json_input, dict):
        for k, v in json_input.items():
            if k == lookup_key:
                yield v
            else:
                yield from item_generator(v, lookup_key)
    elif isinstance(json_input, list):
        for item in json_input:
            yield from item_generator(item, lookup_key)

script_start_time = time.perf_counter()

#Pre processing:
#Set the Android platforms file, the APK folder and the output file

#Remote path
prefix_path = '/data/PRICEExperiments/'

#apk_dir = f'{prefix_path}DomainApps'
apk_dir = f'{prefix_path}AllRQ2Apps'
#apk_dir = f'{prefix_path}ECommerceAndShopping'
#apk_dir = f'{prefix_path}MessagingAndSocialMedia'
#apk_dir = f'{prefix_path}BankingAndFinance'
#apk_dir = f'{prefix_path}SportsAndGames'
#apk_dir = f'{prefix_path}TechAndEducation'
#apk_dir = f'{prefix_path}HealthAndFitness'
#apk_dir = f'{prefix_path}NewsAndEntertainment'
#apk_dir = f'{prefix_path}Others'

#Get the path of the directory containing this script
current_dir = os.path.dirname(os.path.abspath(__file__))
platforms_dir = f'{current_dir}/platforms'
input_dir = f'{current_dir}/input'
output_dir = f'{current_dir}/output'
finaloutput_dir = f'{current_dir}/output/RQ2'
error_csv = f'{current_dir}/stats/RQ2/finalerror.csv'
apk_files = find_all_files(apk_dir, 'apk')
csv_file=f'{current_dir}/stats/RQ2/stats.csv'
if(os.path.isfile(csv_file)):
    print("Resume processing")
else:
    initialiseCSVFile(csv_file)

#Get all APK files in the given folder and iterate over them recursively
if apk_files is not None:
    for f in apk_files:
        error = 0
        app_name = os.path.basename(f)
        config_files = find_all_files(input_dir, 'yaml')
        if config_files is not None:
            for c in config_files:
                config_file=c
        else:
            print('Error: no config file given as input.')
        print('....................................Started processing the application:',app_name,'....................................')
        #Processing: 
        #Use PrivacyFX to extract all risky system API sources.
        orderedJsonOutput = f'{finaloutput_dir}/{app_name}.json'
        #If app has already been analyzed, don't analyze again.
        if(os.path.isfile(orderedJsonOutput)):
                print("App has already been analyzed. Won't analyze again.")
                process_start_time = time.perf_counter()
        else:
            process_start_time = time.perf_counter()
            print("Command executed:")
            print("java -jar PrivacyFX.jar "+f +" " +finaloutput_dir + "/ "+config_file +" "+platforms_dir)
            #Use the following commannd to achieve timeouts of 10 minutes for all apps
            os.system("timeout 10m java -jar PrivacyFX.jar "+f +" " +finaloutput_dir + "/ "+config_file +" "+platforms_dir)
            #default output: "output/privacySources.json
            #Change this default output file whenever needed
            process_end_time = time.perf_counter()
            print('....................................Finished processing the application:',app_name,'....................................')
            print('Time taken:',process_end_time - process_start_time,' seconds')
            postprocess_start_time = time.perf_counter()
            #Post processing: 
            #If the output json file exists, order the risk categories according to their danger
            jsonOutput = f'{output_dir}/privacySources.json'
            if os.path.isfile(jsonOutput):
                #Successful static analysis
                with open(jsonOutput) as f:
                    json_data = json.load(f)
                sorted_json_data = json.dumps(json_data, sort_keys=True, indent=2)
                with open(orderedJsonOutput, "w") as outfile:
                    outfile.write(sorted_json_data)
                subprocess.run(["rm", jsonOutput])
                privacyfx_wp_output = f'{finaloutput_dir}/wp.csv'
                #We don't need this file.
                subprocess.run(["rm", privacyfx_wp_output]);
                
                postprocess_end_time = time.perf_counter()
                data=[app_name, process_end_time - process_start_time, postprocess_end_time - postprocess_start_time]
                #Write the statistics
                with open(csv_file, 'a', encoding='UTF8', newline='') as f:
                    writer = csv.writer(f)
                    writer.writerow(data)
            else: #Output file not generated
                print('Error: App', app_name,'could not be analyzed.')
                error = error + 1
                with open(error_csv, 'a', encoding='UTF8', newline='') as f:
                    writer = csv.writer(f)
                    writer.writerow([app_name])
else:
    print('Error: no APKs given as input.')
script_end_time = time.perf_counter()
print('Total time taken:',(script_end_time - script_start_time)/60,'minutes')
